package pl.spjava.demospringboot.interceptors;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;

import java.util.logging.Level;
import java.util.logging.Logger;

@Aspect
@Component
public class MethodCallLoggingInterceptor {
    private static final Logger logger = Logger.getLogger(MethodCallLoggingInterceptor.class.getName());

    // This pointcut defines which methods are affected by this interceptor
    // First  part: methods explicitly annotated with @MethodCallLogged
    // Second part: methods from classes annotated with @MethodCallLogged
    @Pointcut("execution(@pl.spjava.demospringboot.interceptors.MethodCallLogged * *(..)) " +
        "|| within(@pl.spjava.demospringboot.interceptors.MethodCallLogged *)")
    public void methodCallLoggedAnnotatedMethods() {}

    // This advice applies around invocations of methods defined by pointcut above
    @Around("methodCallLoggedAnnotatedMethods()")
    public Object getAccountOperationInfo(ProceedingJoinPoint joinPoint) throws Throwable{

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        String username = (authentication != null) ? authentication.getName() : "--ANONYMOUS--";

        StringBuilder message = new StringBuilder("Method call: ");
        Object result;
        try {
            try {
                message.append(joinPoint.getSignature().getDeclaringTypeName());
                message.append(".");
                message.append(joinPoint.getSignature().getName());
                message.append("| user: ").append(username);
                message.append("| args: ");
                    for (Object arg : joinPoint.getArgs()) {
                        message.append(String.valueOf(arg)).append(" ");
                    }
            } catch (Exception e) {
                logger.log(Level.SEVERE, "| Unexpected exception within interceptor: ", e);
                throw e;
            }

            result = joinPoint.proceed();

        } catch (Throwable t) {
            message.append("| thrown exception: ").append(t.toString());
            logger.log(Level.SEVERE, message.toString(), t);
            throw t;
        }

        message.append("| returned: ").append(String.valueOf(result)).append(" ");

        logger.info(message.toString());

        return result;

    }

}