package pl.spjava.demospringboot.rest;

import jakarta.validation.Valid;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import pl.spjava.demospringboot.converters.PersonConverter;
import pl.spjava.demospringboot.dto.PersonDTO;
import pl.spjava.demospringboot.dto.PersonUpdateCmdDTO;
import pl.spjava.demospringboot.interceptors.MethodCallLogged;
import pl.spjava.demospringboot.services.PersonService;

import java.util.List;
import java.util.NoSuchElementException;
import java.util.UUID;

@RestController
@MethodCallLogged
@RequestMapping(value = "/persons", produces = MediaType.APPLICATION_JSON_VALUE)
@RequiredArgsConstructor
@PreAuthorize("denyAll()")
public class PersonsController {

    @NonNull
    private final PersonService personService;
    // Injection is done by constructor generated by @RequiredArgsConstructor. This allows field to be final and makes mocking easier.
    // In Spring, if there's a constructor for given field, and the field's class is a component, you don't even need to mark the constructor parameter as @Autowired


    @GetMapping("")
    @PreAuthorize("permitAll()") //to be secured in future!
    public List<PersonDTO> getAllPersons() {
        return PersonConverter.fromPersonEntities(personService.getAllPersons());
    }

    @GetMapping("/{id}")
    @PreAuthorize("permitAll()") //to be secured in future!
    public PersonDTO getPerson(@PathVariable UUID id) {
        return PersonConverter.fromPersonEntity(personService.getPerson(id));
    }

    @GetMapping("/login/{login}")
    @PreAuthorize("permitAll()") //to be secured in future!
    public PersonDTO findPersonByLogin(@PathVariable String login) {
        return PersonConverter.fromPersonEntity(personService.findPersonByLogin(login));
    }

    @GetMapping("/self")
    @PreAuthorize("hasAnyRole({'ADMIN','EMPLOYEE'})")
    public PersonDTO findSelfPerson() {
        return PersonConverter.fromPersonEntity(personService.findPersonByLogin(
                SecurityContextHolder.getContext().getAuthentication().getName()
        ));
    }

    @PostMapping(value = "", consumes = MediaType.APPLICATION_JSON_VALUE)
    @PreAuthorize("permitAll()") //to be secured in future!
    public PersonDTO createPerson(@RequestBody @Valid PersonDTO newPerson) {
        return PersonConverter.fromPersonEntity(personService.addPerson(PersonConverter.fromPersonDTO(newPerson)));
    }

    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE)
    @PreAuthorize("permitAll()") //to be secured in future!
    public PersonDTO updatePerson(@PathVariable UUID id, @RequestBody @Valid PersonUpdateCmdDTO updateData) {
        return PersonConverter.fromPersonEntity(personService.updatePerson(updateData, id));
    }

    @PostMapping("/{id}/activate")
    @PreAuthorize("permitAll()") //to be secured in future!
    public void activatePerson(@PathVariable UUID id) { personService.activatePerson(id); }

    @PostMapping("/{id}/deactivate")
    @PreAuthorize("permitAll()") //to be secured in future!
    public void deactivatePerson(@PathVariable UUID id) { personService.deactivatePerson(id); }

    @DeleteMapping("/{id}")
    @PreAuthorize("hasRole('ADMIN')")
    public void removePerson(@PathVariable UUID id) { personService.removePerson(id); }

    // You can place specific ("private") exception handler directly inside your controller.
    // It disables @ControllerAdvice handlers for that exception
//    @ExceptionHandler(NoSuchElementException.class)
//    @ResponseStatus(HttpStatus.NOT_FOUND)
//    public ResponseEntity<String> handleNoSuchElementException(NoSuchElementException nsee)
//    {
//        return ResponseEntity.status(HttpStatus.NOT_FOUND).body("CUSTOMIZED "+nsee.getMessage());
//    }
}
