package pl.spjava.demospringboot.rest.handlers;

import jakarta.validation.ConstraintViolation;
import jakarta.validation.ConstraintViolationException;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.HttpStatusCode;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authorization.AuthorizationDeniedException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import java.util.NoSuchElementException;

@ControllerAdvice
public class GeneralControllerExceptionHandler extends ResponseEntityExceptionHandler {


    // For better validation error responses (i.e. multiple error descriptions)
    // You can't handle it by try-catch on controller methods, since validation is performed before method is called
    // Comment it out to see how Spring deals with validation errors by default
    // https://reflectoring.io/bean-validation-with-spring-boot/
    @ExceptionHandler(ConstraintViolationException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    @ResponseBody
    ValidationErrorResponse onConstraintValidationException(
            ConstraintViolationException e) {
        ValidationErrorResponse error = new ValidationErrorResponse();
        for (ConstraintViolation violation : e.getConstraintViolations()) {
            error.getViolations().add(
                    new ValidationErrorResponse.Violation(violation.getPropertyPath().toString(), violation.getMessage()));
        }
        return error;
    }

    // For better validation error responses (i.e. multiple error descriptions)
    // You can't handle it by try-catch on controller methods, since validation is performed before method is called
    // Comment it out to see how Spring deals with validation errors by default
    // https://reflectoring.io/bean-validation-with-spring-boot/
    // Hovewer, you cannot define your brand new method for this, because you're going to get
    // That's because there's a method in ResponseEntityExceptionHandler already, so you have to @Override it
//    @ExceptionHandler(MethodArgumentNotValidException.class)
//    @ResponseStatus(HttpStatus.BAD_REQUEST)
//    @ResponseBody
    @Override
    protected ResponseEntity<Object> handleMethodArgumentNotValid(MethodArgumentNotValidException ex,
                                                                  HttpHeaders headers,
                                                                  HttpStatusCode status,
                                                                  WebRequest request) {
        ValidationErrorResponse error = new ValidationErrorResponse();
        for (FieldError fieldError : ex.getBindingResult().getFieldErrors()) {
            error.getViolations().add(
                    new ValidationErrorResponse.Violation(fieldError.getField(), fieldError.getDefaultMessage()));
        }
        return ResponseEntity.ofNullable(error);
    }

    // For method level authorization exceptions handling
    @ExceptionHandler(AuthorizationDeniedException.class)
    @ResponseStatus(HttpStatus.FORBIDDEN)
    public ResponseEntity<Object> handleAuthorizationException(
            RuntimeException exception,
            WebRequest request
    ){
        return ResponseEntity.status(HttpStatus.FORBIDDEN).body("Authorization exception: "+exception.getMessage());
    }

   // For general unknown exceptions handling
    @ExceptionHandler(RuntimeException.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public ResponseEntity<Object> handleAllUncaughtException(
            RuntimeException exception,
            WebRequest request
    ){
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Exception thrown by controller: "+exception.getMessage());
    }

//    @Override
//    public ResponseEntity<Object> handleExceptionInternal(
//            Exception ex,
//            Object body,
//            HttpHeaders headers,
//            HttpStatus status,
//            WebRequest request) {
//
//        return buildErrorResponse(ex,status,request);
//    }

}