package pl.spjava.demospringboot.security;

import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.JWSAlgorithm;
import com.nimbusds.jose.JWSHeader;
import com.nimbusds.jose.JWSSigner;
import com.nimbusds.jose.JWSVerifier;
import com.nimbusds.jose.crypto.MACSigner;
import com.nimbusds.jose.crypto.MACVerifier;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.SignedJWT;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

@Component
public class JWTGeneratorVerifier {


    private @Value("${jwt.secret}") String SECRET;
    private @Value("${spring.application.name}") String ISSUER;
    private @Value("${jwt.timeout}") long JWT_TIMEOUT_MS;

    public String generateJWTString(String subject, List<String> authGroups) {
        try {
            // Create HMAC signer
            JWSSigner signer = new MACSigner(SECRET);

            // Prepare claimset
            JWTClaimsSet.Builder builder = new JWTClaimsSet.Builder()
                    .subject(subject)
                    .issuer(ISSUER)
                    .claim("auth", authGroups)
                    .expirationTime(new Date(new Date().getTime() + JWT_TIMEOUT_MS));

//            for(String authGroup:authGroups)
//                builder = builder.claim("auth", authGroup);

            SignedJWT signedJWT = new SignedJWT(new JWSHeader(JWSAlgorithm.HS256), builder.build());

            // Apply the HMAC protection
            // We do NOT encrypt the JWT token, just make it credible. The client probably needs the information on its identity and access level(s)
            // Use https to secure the JWT during transmission
            signedJWT.sign(signer);

            return signedJWT.serialize();

        } catch (JOSEException jex) {
            Logger.getLogger(JWTGeneratorVerifier.class.getName()).log(Level.SEVERE, null, jex);
            return "JWT failure";
        }

    }

    public boolean validateJWTSignature(String tagValue) {
        // this is ONLY validation of the signature itself
        if (null == tagValue || tagValue.isEmpty()) return false;
        try {
            SignedJWT jwtToken = SignedJWT.parse(tagValue);
            JWSVerifier verifier = new MACVerifier(SECRET);
            Logger.getLogger(JWTGeneratorVerifier.class.getName()).log(Level.INFO, "JWT token: {0} validation result: {1}", new Object[]{jwtToken.getParsedString(),jwtToken.verify(verifier)});

            return jwtToken.verify(verifier);
        } catch (ParseException | JOSEException ex) {
            Logger.getLogger(JWTGeneratorVerifier.class.getName()).log(Level.SEVERE, null, ex);
            return false; //assume worst scenario!
        }
    }

}
