package pl.spjava.demospringboot.security;

import com.nimbusds.jwt.SignedJWT;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

@Component
@RequiredArgsConstructor
public class JWTRequestFilter extends OncePerRequestFilter {

    private static final Logger LOG=Logger.getLogger(JWTRequestFilter.class.getName());
    @NonNull
    private final JWTGeneratorVerifier jwtGeneratorVerifier;
    // Injection is done by constructor generated by @RequiredArgsConstructor. This allows field to be final and makes mocking easier.
    // In Spring, if there's a constructor for given field, and the field's class is a component, you don't even need to mark the constructor parameter as @Autowired
    @Override
    protected void doFilterInternal(HttpServletRequest request, @NonNull HttpServletResponse response, @NonNull FilterChain filterChain)
            throws ServletException, IOException {

        final String authorizationHeader = request.getHeader("Authorization");

        // JWT Token is in the form "Bearer token". Remove Bearer word and get only the Token
        if (authorizationHeader != null && authorizationHeader.startsWith("Bearer ")) {
            String authToken = authorizationHeader.substring("Bearer ".length());
            if (jwtGeneratorVerifier.validateJWTSignature(authToken)) {
                try {
                    SignedJWT jwtToken = SignedJWT.parse(authToken);
                    String login = jwtToken.getJWTClaimsSet().getSubject();
                    List<Object> groups = jwtToken.getJWTClaimsSet().getListClaim("auth");
                    Date expirationTime = jwtToken.getJWTClaimsSet().getExpirationTime();
                    boolean tokenExpired = new Date().after(expirationTime);
                    LOG.log(Level.FINE, "JWT Token info: login: {0} groups: {1} expires: {2} expired: {3}", new Object[]{login, groups, expirationTime, tokenExpired});

                    // Now we know that token is valid. If it isn't expired, we may pass user details to security context
                    if (!tokenExpired) {
                        List<GrantedAuthority> authorities = new ArrayList<>();
                        // NOTE that if an authority read from token is to be interpreted as a role (ie. to be checked with @PreAuthorize("hasRole('...'))
                        for (Object group:groups) authorities.add(new SimpleGrantedAuthority("ROLE_"+String.valueOf(group)));
                        SecurityContextHolder.getContext().setAuthentication(new UsernamePasswordAuthenticationToken(login, null, authorities));
                    }
                } catch (ParseException ex) { //That shouldn't happen after validation!
                    LOG.log(Level.FINE, "JWT token parsing: ", ex);
                }
            }
        }
        filterChain.doFilter(request, response);
    }
}
