package pl.spjava.demospringboot.services;

import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Sort;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import pl.spjava.demospringboot.dto.PersonDTO;
import pl.spjava.demospringboot.dto.PersonUpdateCmdDTO;
import pl.spjava.demospringboot.entities.PersonEntity;
import pl.spjava.demospringboot.exceptions.ConcurrentUpdateException;
import pl.spjava.demospringboot.exceptions.DeleteActivePersonException;
import pl.spjava.demospringboot.exceptions.PersonNotFoundException;
import pl.spjava.demospringboot.interceptors.MethodCallLogged;
import pl.spjava.demospringboot.repositories.PersonRepository;

import java.util.List;
import java.util.UUID;

@Component
@MethodCallLogged
@RequiredArgsConstructor
@Service // It's nothing more than decoration indicating class' purpose
@Transactional(propagation = Propagation.REQUIRES_NEW)
@PreAuthorize("denyAll()")
public class PersonService {

    /* How is it possible? We didn't define any implementation of PersonRepository!
        This is SpringData magic. Based on interface declaration SpringData generates an implementation at runtime
        and injects it through Lombok's @RequiredArgsConstructor (note @NonNull on that field!).
     */
    @NonNull
    private final PersonRepository personRepository;

    @PreAuthorize("permitAll()") //to be secured in future!
    public List<PersonEntity> getAllPersons() {
        return personRepository.findAll(Sort.unsorted());
    }

    @PreAuthorize("permitAll()") //to be secured in future!
    public PersonEntity addPerson(PersonEntity newPerson) {
        return personRepository.save(newPerson);
    }

    @PreAuthorize("permitAll()") //to be secured in future!
    public PersonEntity updatePerson(PersonUpdateCmdDTO updateData, UUID id) {
        PersonEntity updatedPerson = getPerson(id);

        if(updateData.getVersion() < updatedPerson.getVersion())
            throw new ConcurrentUpdateException();

        if(null != updateData.getEmail()) updatedPerson.setEmail(updateData.getEmail());
        if(null != updateData.getAge()) updatedPerson.setAge(updateData.getAge());

        return personRepository.save(updatedPerson); // Just to save changes to the database and update object's version before it gets returned
    }

    @PreAuthorize("permitAll()") //to be secured in future!
    public PersonEntity getPerson(UUID id) {
        return personRepository.findById(id).orElseThrow(PersonNotFoundException::new);
    }

    @PreAuthorize("permitAll()") //to be secured in future!
    public void activatePerson(UUID id) {
        setPersonActivity(id, true);
    }

    @PreAuthorize("permitAll()") //to be secured in future!
    public void deactivatePerson(UUID id) {
        setPersonActivity(id, false);
    }

    private void setPersonActivity(UUID id, boolean active) {
        getPerson(id).setActive(active); // doesn't work until you set transaction boundaries with @Transactional
//        PersonEntity modifiedPerson = getPerson(id);
//        modifiedPerson.setActive(active);
//        personRepository.save(modifiedPerson); // We have to use save() only if there's no @Transactional
    }

    @PreAuthorize("hasRole('ADMIN')")
    public void removePerson(UUID id) {
        PersonEntity personToBeDeleted = getPerson(id);
        if(personToBeDeleted.isActive()) throw new DeleteActivePersonException();
        personRepository.deleteById(id);
    }

    @PreAuthorize("permitAll()") //to be secured in future!
    public PersonEntity findPersonByLogin(String login) {
        return personRepository.findByLogin(login);
    }
}
